(* Copyright (c) Mark J. Kilgard, 1994. *)

(* (c) Copyright 1993, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * US Government Users Restricted Rights
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 *)

(*  polys.ml
 *  This program demonstrates polygon stippling.
 *)
open GL
open Glu
open Glut

let display() =
  let fly =
    Bigarray.Array1.of_array Bigarray.int8_unsigned Bigarray.c_layout [|
      0x00; 0x00; 0x00; 0x00; 0x00; 0x00; 0x00; 0x00; 0x03; 0x80; 0x01; 0xC0;
      0x06; 0xC0; 0x03; 0x60; 0x04; 0x60; 0x06; 0x20; 0x04; 0x30; 0x0C; 0x20;
      0x04; 0x18; 0x18; 0x20; 0x04; 0x0C; 0x30; 0x20; 0x04; 0x06; 0x60; 0x20;
      0x44; 0x03; 0xC0; 0x22; 0x44; 0x01; 0x80; 0x22; 0x44; 0x01; 0x80; 0x22;
      0x44; 0x01; 0x80; 0x22; 0x44; 0x01; 0x80; 0x22; 0x44; 0x01; 0x80; 0x22;
      0x44; 0x01; 0x80; 0x22; 0x66; 0x01; 0x80; 0x66; 0x33; 0x01; 0x80; 0xCC;
      0x19; 0x81; 0x81; 0x98; 0x0C; 0xC1; 0x83; 0x30; 0x07; 0xe1; 0x87; 0xe0;
      0x03; 0x3f; 0xfc; 0xc0; 0x03; 0x31; 0x8c; 0xc0; 0x03; 0x33; 0xcc; 0xc0;
      0x06; 0x64; 0x26; 0x60; 0x0c; 0xcc; 0x33; 0x30; 0x18; 0xcc; 0x33; 0x18;
      0x10; 0xc4; 0x23; 0x08; 0x10; 0x63; 0xC6; 0x08; 0x10; 0x30; 0x0c; 0x08;
      0x10; 0x18; 0x18; 0x08; 0x10; 0x00; 0x00; 0x08 |] in

  let halftone =
    Bigarray.Array1.of_array Bigarray.int8_unsigned Bigarray.c_layout [|
      0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA;
      0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55;
      0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA;
      0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55;
      0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA;
      0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55;
      0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA;
      0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55;
      0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA;
      0x55; 0x55; 0x55; 0x55; 0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55;
      0xAA; 0xAA; 0xAA; 0xAA; 0x55; 0x55; 0x55; 0x55 |] in

  glClear [GL_COLOR_BUFFER_BIT];

  (* draw all polygons in white *)
  glColor3 1.0 1.0 1.0;

  (* draw one solid, unstippled rectangle, *)
  (* then two stippled rectangles          *)
  glRect 25.0 25.0 125.0 125.0;
  glEnable GL_POLYGON_STIPPLE;
  glPolygonStipple fly;
  glRect 125.0 25.0 225.0 125.0;
  glPolygonStipple halftone;
  glRect 225.0 25.0 325.0 125.0;
  glDisable GL_POLYGON_STIPPLE;

  glFlush ();
;;

let myinit () =
(*  clear background to black   *)
  glClearColor 0.0 0.0 0.0 0.0;
  glShadeModel GL_FLAT;
;;


let reshape ~width:w ~height:h =
  glViewport 0 0 w h;
  glMatrixMode GL_PROJECTION;
  glLoadIdentity();
  glOrtho 0.0 (float w) 0.0 (float h) (-1.0) (1.0);
  glMatrixMode GL_MODELVIEW;
  glLoadIdentity();
;;


let keyboard ~key ~x ~y =
  begin match key with
  | '\027' ->  (* Escape *)
      exit(0);
  | _ -> ()
  end;
  glutPostRedisplay();
;;


(*  Main Loop
 *  Open window with initial window size, title bar,
 *  RGBA display mode, and handle input events.
 *)
let () =
  let _ = glutInit Sys.argv in
  glutInitDisplayMode [GLUT_SINGLE; GLUT_RGB];
  glutInitWindowSize 350 150;
  let _ = glutCreateWindow Sys.argv.(0) in
  myinit ();
  glutDisplayFunc ~display;
  glutReshapeFunc ~reshape;
  glutKeyboardFunc ~keyboard;
  glutMainLoop();
;;

